%% (IEB) state the endogenouse variables; 'q', job-filing rate; 's', job
%% finding rate; 'n', employment rate; 'u', portion of 
%% job-seekers at the start of each period; 'ur', unemployment rate; 'v', 
%%portion of vacancies; 'wr', real wage; 'wrc', wage adjustment cost; 'dwrc', 
%% its derivative; 'piew', wage inflation; 'ytot' final total output/production; 
%% 'y', output/production per employee;'k', capital; 'rk', output per capita; 
%% 'inv', investment; 'pie', inflation; 'c', consumption; 'tau', time-varying 
%% term, endogenous part of worker's bargaining power; 'omega', worker's 
%% effective bargaining power; 'curlphi', relative cost of intermediate goods;
%%'gampi', the price adjustment cost; 'DGampi', its derivative; nint, nominal 
%% interest rate.

addpath(genpath('supportfiles'));

var 
q, s, n, u, ur, v, wr,               //labour market
    wrc, dwrc, piew, ytot, y, k, rk, inv, pie, c, tau, omega, 
    DAI, IAC, TobQ, // adjustment costs on investment
    DAC, AC, // adjustment costs on employment
    curlphi, gampi, DGampi, nint, lambda, 
    z, risk                    //shocktypes
    ;
%% (IEB) state the exogenous variabes (e_r, risk premium; e_m, monetary policy;
%% e_z, technology)
varexo 
e_r 
e_m 
e_z;

%% (IEB) state the paramters
parameters  
g %% IEB: growth rate
beta %% IEB: household discount rate
b 
alpha %% IEB:Production function curvature
delta %% IEB: capital depreciation rate
omegar %% interest rate smoothing
omegapi %% IEB: response coefficient to deviation of inflation rate target 
omegau %% IEB: response coefficient to deviation of output growth from trend growth
theta %% IEB: matching function elasticity (labor market)
eta %% IEB: exogenous bargaining power of employees
nu %% IEB: indexation of wage changes (as stated by Abritti & Fahr in 
%% 'dwparam)small_baseline.m').
psi %% IEB: 'Mapping Calvo probability of Rotemberg adjustment costs' (as stated
% by Abritti & Fahr in 'dwparam)small_baseline.m')
epsilon %% IEB: elasticity of substitution between goods
chiw %% IEB: quadratic component of wage adjustment cost
psiw %% IEB: degree of asymmetry of wage adjustment cost
ADI %% IEB: quadratic component of investment adjustment cost
deltaprime 
psii // adjustment costs on investment %%(IEB: degree of asymmetry)
chin // adjustment costs on employment %%(IEB: quadratice component)
psin    // adjustment costs on employment %%(IEB: degree of asymmetry)
Nss %% IEB: Steady state employment
Yss    %%IEB: steady state production
roz %% IEB: persistance of technology shock
stdez %% IEB: standard deviation of technology shock
stdem %% IEB: standard deviation of monetary policy shock
rorisk %% IEB: persistance of risk premium shocks
stderisk  %% IEB: standard deviation of risk premium shocks
sigmam %% IEB: matching function constant
kap %% IEB: vacancy posting costs
rho %% IEB: exogenous job-destruction rate
; 


// Different calibrations:

// Baseline calibration with dnwr
@# include "dwparam_small_baseline.m" 

// Calibration withOUT dnwr
// @# include "dwparam_small_noDWR.m" 

// Calibration with asymmetric adjustment costs on employment
// @# include "dwparam_small_EAC.m"

// Calibration with asymmetric adjrun(ustment costs on investment 
// @# include "dwparam_small_IAC.m" 

// Baseline calibration with dnwr and US calibration
// @# include "dwparam_small_US.m" 

// Baseline calibration with dnwr and accomodative mon.policy
// @# include "dwparam_small_MP.m" 

%% (IEB) include initial values for simulations
initval;
lambda  = lambdass;
tau     = 1;
omega   = eta;
AC      = 0;
DAC     = 0;
TobQ    = 1;
IAC     = 0;
DAI     = 0;
inv     = Iss;
k       = kss;
n       = Nss;
v       = vss;
u       = 1-(1-rhoss)*n; 
ur      = 1-n; 
q       = qss;
s       = sss;
wr      = Wss;
c       = Css;
ytot    = Yss;
y       = alpha*ytot/n ;
curlphi = (epsilon-1)/epsilon;
rk      = RKss;
pie     = piess;
piew    = piewss;
gampi   = 0;
DGampi  = 0;
wrc     = 0;
dwrc    = 0;
nint    = Rss;

z       = 0;
risk     = 0;
e_m     = 0;
end;

%% (IEB) begin defining the equations in the model
model;

//Labour market
  q  = sigmam* (v/u)^(-theta);
  s  = sigmam*(v/u)^(1-theta);
  n  = (1-rho)*n(-1) + (v*q);
  u  = 1 - (1-rho)*n(-1);
  ur = 1-n;

  kap/lambda/q  = curlphi*y - wr*(1+wrc)-DAC/n(-1) + (1-rho)*beta*lambda(+1)/lambda * kap/lambda(+1)/q(+1) + beta*lambda(+1)/lambda * DAC(+1)*n(+1)/(n^2);

// Wages
  tau = 1 + wrc + dwrc*piew/pie^nu 
      - (1-rho) * beta*lambda(+1)/lambda/g * dwrc(+1)*(piew(+1))^2/(pie(+1))^(1+nu);

  omega = eta/(eta+(1-eta)*tau);

  wr*(1+omega*wrc) = omega*(curlphi*y-DAC/n(-1) + (1-rho)* beta*lambda(+1)/lambda * kap/lambda(+1)/q(+1) * (1-(1-s(+1))*(1-omega)*omega(+1)/((1-omega(+1))*omega)))
                       +omega*beta*lambda(+1)/lambda * DAC(+1)*n(+1)/(n^2) + (1-omega)*b;

//////////////////////////////////////////////////////////////////////////
//Employment adjustment costs 
// Quadratic + Linex
   AC  - (chin-1)/2*(n/n(-1)-1)^2 - 1/psin^2*(  exp(-psin*(n/n(-1)-1)) + psin*(n/n(-1)-1)-1);
   DAC  - (chin-1)  *(n/n(-1)-1)   - 1/psin  *(1-exp(-psin*(n/n(-1)-1))); 

//////////////////////////////////////////////////////////////////////////
//Wage adjustment costs 
// Quadratic + Linex
   wrc  - (chiw-1)/2*(piew/pie^nu-g)^2 - 1/psiw^2*(  exp(-psiw*(piew/pie^nu-g)) + psiw*(piew/pie^nu-g)-1);
  dwrc  - (chiw-1)  *(piew/pie^nu-g)   - 1/psiw  *(1-exp(-psiw*(piew/pie^nu-g)) );

//////////////////////////////////////////////////////////////////////////
//Production
  ytot  = exp(z)*(n)^alpha * k^(1-alpha); %MISSING gamma? 
  y     = alpha*ytot/n; 
  rk    = curlphi * (1-alpha)*ytot/k;   

//Capital
  k   = (1-delta) * k(-1) + inv; % WAS:k - (1-delta)*k(-1)/g - inv

//////////////////////////////////////////////////////////////////////////
// Investment Adjustment costs

  IAC - (ADI-1)/2*(inv/inv(-1)-1)^2 - 1/psii^2*(  exp(-psii*(inv/inv(-1)-1)) + psii*(inv/inv(-1)-1)-1);
  DAI  - (ADI-1)  *(inv/inv(-1)-1)   - 1/psii  *(1-exp(-psii*(inv/inv(-1)-1)));
  TobQ =1 + IAC + DAI*inv/inv(-1) - beta*(lambda(+1)/lambda)*DAI(+1)*(inv(+1)/inv)^2;
  TobQ= rk + beta*lambda(+1)/lambda/g*(TobQ(+1)*(1-delta));

//////////////////////////////////////////////////////////////////////////
// Resource constraint

  c + inv = ytot*(1-gampi) - wrc*wr*n - IAC*inv - AC;

// Euler & Lagrange multiplier
  lambda = beta*nint*exp(risk)*lambda(+1)/g/pie(+1);
  lambda = 1/c;

//Price Adjustment costs & Inflation Philips curve 
  DGampi*pie = epsilon*(curlphi+gampi)-(epsilon-1) + beta*lambda(+1)/lambda* ytot(+1)/ytot * DGampi(+1) * pie(+1);
   gampi = psi/2 * (pie-1)^2;
  DGampi = psi   * (pie-1);

// Wage identity  
  piew = g*(wr/wr(-1))*pie;
  %When I removed g, it was not possible to calculate SS

//Taylor rule reacting to inflation and output growth
%IEB NOTE: g/beta refers to the natural interest rate (growth rate/discount factor)
  nint   = exp(-stdem*e_m) * (nint(-1))^omegar * (g/beta * pie^omegapi * (ytot/ytot(-1))^omegau)^(1-omegar);

// Shock process
  z     =             roz  *   z(-1) + stdez * e_z;  
  risk   =             rorisk  * risk(-1) + stderisk * (e_r);
end;

shocks;
    var 
    e_m = 0.0001;   // monetary
    var 
    e_r = 0.0001;   // risk premium
    var 
    e_z = 0.0001;   // transitory technology
end;

steady;

addnames('ytot',   'Production');
addnames('pie',    'Price inflation');
addnames('piew',   'Wage inflation');
addnames('ur',     'Unemployment rate');
addnames('wr',     'Real wage');
addnames('n',      'Employment');
addnames('q',      'Job filling rate');
addnames('v',      'Vacancies');
addnames('nint',   'Nominal interest rate');
addnames('inv',    'Investment');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%    Common block for all parts      %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

options_.pruning    = 1;
options_.relative_irf= 1;
options_.nocorr     = 1;
options_.nomoments  = 1;
options_.noprint    = 1;
options_.simul_seed = 123123;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%        Initialization                           %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

stoch_simul(periods=566,irf=0) %%INJI AMMENDED: Periods was set to 1000
                                % and irf was 0; irf=15 enables results 
                                %to show for 15 periods after the shock
ytot 
pie 
piew 
ur 
wr 
n 
inv 
nint 
v;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%        Second moments and skewness              %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dwmoments

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%        Harding and Pagan Analysis on selected variables;     %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dw_turningpoints

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%        Impulse responses                                     %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

myirfreplic     = 1000; %IEB: changed to 1000 from 300, for 1000 simulations
myirflength     = 15;
myirfstartval   = 0;
momreplic       = 1;
modname1        = strcat('DNWR');

varirf      = strvcat('piew','inv','v','pie','n','ur','wr','ytot','nint');
num_vars    =size(varirf,1); %IEB: Added this line

%IEB: Added section below to crease zero arrays to store IRFs from simualtions
if myirfstartval==0
        irf_pos=zeros(myirflength+1, num_vars, myirfreplic); %for +ve shocks
        irf_neg=zeros(myirflength+1, num_vars, myirfreplic); %for -ve shocks
else
        irf_pos=zeros(myirflength, num_vars, myirfreplic); %for +ve shocks
        irf_neg=zeros(myirflength, num_vars, myirfreplic); %for -ve shocks
end
% Technology shock
myshocktypes    = ['e_z']; 
myshocksize     = [-2;2];
%IEB: WAS dynareirf(myshocktypes,myirfreplic,myirflength,varirf,myshocksize,
                   %myirfstartval,modname1)

%IEB: added section below for computing IRFs 
for rep=1:myirfreplic
    irf_pos(:, :, rep) = dynareirf(M_,options_,oo_,myshocktypes, 2, myirflength, varirf, myshocksize(2), myirfstartval, modname1)';
    
    irf_neg(:, :, rep) = dynareirf(M_,options_,oo_,myshocktypes, 1, myirflength, varirf, myshocksize(1), myirfstartval, modname1)';
end

%IEB: added section below to create average of IRFs across simulations
avg_irf_pos_tech = mean(irf_pos, 3)
avg_irf_neg_tech = mean(irf_neg, 3)

%IEB: added section below to plot average IRF across simulations
figure;
for i= 1:num_vars
    subplot(3, 3, i); 
    plot(1:myirflength, avg_irf_pos_tech(:, i), 'b', 'LineWidth', 1.5); hold on;
    plot(1:myirflength, avg_irf_neg_tech(:, i), 'r', 'LineWidth', 1.5);
    title(varirf(i, :));
    xlabel('');
    ylabel('');
    legend('2\sigma', '-2\sigma');
    hold off;
end
suptitle('Technology Shock');

% Risk premium shock
myshocktypes    = ['e_r']; 
myshocksize     = [-2.5;2.5];
%IEB: WAS - dynareirf(myshocktypes,myirfreplic,myirflength,varirf,
                      %myshocksize,myirfstartval,modname1)

%IEB: added section below for computing IRFs                       
for rep = 1:myirfreplic
 
    irf_pos(:, :, rep) = dynareirf(myshocktypes, 1, myirflength, varirf, myshocksize(1), myirfstartval, modname1);
    
    irf_neg(:, :, rep) = dynareirf(myshocktypes, 2, myirflength, varirf, myshocksize(2), myirfstartval, modname1);
end

%IEB: added section below to create average of IRFs across simulations
avg_irf_pos_risk= mean(irf_pos, 3);
avg_irf_neg_risk = mean(irf_neg, 3);

%IEB: added section below to plot average IRF across simulations
figure;
for i= 1:num_vars
    subplot(3, 3, i); 
    plot(1:myirflength, avg_irf_pos_risk(:, i), 'b', 'LineWidth', 1.5); hold on;
    plot(1:myirflength, avg_irf_neg_risk(:, i), 'r', 'LineWidth', 1.5);
    title(varirf(i, :));
    xlabel('');
    ylabel('');
    legend('-2\sigma', '2\sigma');
    hold off;
end
suptitle('Risk Premium Shock');

% Monetary policy shock
myshocktypes    = ['e_m']; 
myshocksize     = [-2.5;2.5];
%IEB: WAS - dynareirf(myshocktypes,myirfreplic,myirflength,varirf,
%myshocksize,myirfstartval,modname1)

%IEB: added section below for computing IRFs                       
for rep = 1:myirfreplic
 
    irf_pos(:, :, rep) = dynareirf(myshocktypes, 1, myirflength, varirf, myshocksize(1), myirfstartval, modname1);
    
    irf_neg(:, :, rep) = dynareirf(myshocktypes, 2, myirflength, varirf, myshocksize(2), myirfstartval, modname1);
end
%IEB: added section below to create average of IRFs across simulations
avg_irf_pos_mp= mean(irf_pos, 3);
avg_irf_neg_mp= mean(irf_neg, 3);

%IEB: added section below to plot average IRF across simulations
figure;
for i= 1:num_vars
    subplot(3, 3, i); 
    plot(1:myirflength, avg_irf_pos_mp(:, i), 'b', 'LineWidth', 1.5); hold on;
    plot(1:myirflength, avg_irf_neg_mp(:, i), 'r', 'LineWidth', 1.5);
    title(varirf(i, :));
    xlabel('');
    ylabel('');
    legend('-2\sigma', '2\sigma');
    hold off;
end
suptitle('Monetary Policy Shock');
